#!/usr/bin/env python

# GOP, a simple OProfile frontend

# Copyright 2005 Andy Grover <andy@groveronline.com>

# This code is released under the GPL version 2.

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import subprocess
import gconf

class MainApp:

	w_main = None
	glade_tree = None
	main_liststore = None
	sub_liststore = None
	prefs = None

	def __init__(self):
		import os
		import gnome.ui

		gnome.init("GOP", "0.2")
		self.glade_tree = gtk.glade.XML("gop.glade", "main_app")
		self.w_main = self.glade_tree.get_widget("main_app")
		self.glade_tree.signal_autoconnect(self)
		self.prefs = Prefs()

		self.main_liststore = gtk.ListStore(str, str, str)

		# setup the main window's treeview
		main_treeview = self.glade_tree.get_widget("main_tree")
		main_treeview.set_model(self.main_liststore)
		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Samples", renderer, text=0)
		main_treeview.append_column(column)
		column = gtk.TreeViewColumn("Percentage", renderer, text=1)
		main_treeview.append_column(column)
		column = gtk.TreeViewColumn("Image", renderer, text=2)
		main_treeview.append_column(column)
		main_tree_sel = main_treeview.get_selection()
		main_tree_sel.connect("changed", self.on_main_row_changed)

		self.sub_liststore = gtk.ListStore(str, str, str)

		# setup the secondary window's treeview
		sub_treeview = self.glade_tree.get_widget("sub_tree")
		sub_treeview.set_model(self.sub_liststore)
		column = gtk.TreeViewColumn("Samples", renderer, text=0)
		sub_treeview.append_column(column)
		column = gtk.TreeViewColumn("Percentage", renderer, text=1)
		sub_treeview.append_column(column)
		column = gtk.TreeViewColumn("Symbol", renderer, text=2)
		sub_treeview.append_column(column)

		self.glade_tree.get_widget("button_stop").set_sensitive(False)

		if os.geteuid() != 0:
			self.glade_tree.get_widget("status_bar").set_status("Not root - can only view results")
			self.glade_tree.get_widget("button_start").set_sensitive(False)
			self.glade_tree.get_widget("button_clear").set_sensitive(False)
			self.glade_tree.get_widget("button_prefs").set_sensitive(False)
			self.glade_tree.get_widget("item_start").set_sensitive(False)
			self.glade_tree.get_widget("item_stop").set_sensitive(False)
			self.glade_tree.get_widget("item_clear").set_sensitive(False)
			self.glade_tree.get_widget("item_prefs").set_sensitive(False)
		else:
			if self.prefs.daemonrc_present == False:
				# create daemonrc with default values
				print "creating daemonrc"
				self.prefs.save()

		self.refresh_main()
		self.w_main.show()

	def refresh_main(self):
		self.main_liststore.clear()
		self.sub_liststore.clear()
		self.main_treedata = self.get_main_tree_data()
		for a in self.main_treedata:
			# only take 1st col and last 2 cols to avoid multiple timers for now
			a = a[0:1] + a[-2:]
			self.main_liststore.append(a)

	def get_main_tree_data(self):
		stdout, stderr = subprocess.Popen(["opreport", "-f", "--no-header"],
		stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
		stdout = stdout.replace("(deleted)", "")
		stdout = stdout.splitlines()
		return [tuple(line.split()) for line in stdout]

	def get_sub_tree_data(self, image):
		cmd = ["opreport", "-l", image, "--no-header"]
		if self.prefs.image_paths:
			cmd.append("-p")
			cmd.append(",".join(self.prefs.image_paths))
		stdout, stderr = subprocess.Popen(cmd,
			stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
		stdout = stdout.replace("(no symbols)", "None")
		stdout = stdout.splitlines()
		return [tuple(line.split()) for line in stdout]

	def on_main_row_changed(self, widget, data=None):
		tree_model, tree_iter = widget.get_selected()
		if not tree_iter:
			return
		name = tree_model[tree_iter][2]
		self.sub_treedata = self.get_sub_tree_data(name)
		self.sub_liststore.clear()
		for a in self.sub_treedata:
			# only want first 2 and last column
			a = a[0:2] + a[-1:]
			self.sub_liststore.append(a)

	def on_start(self, widget, data=None):
		self.glade_tree.get_widget("button_start").set_sensitive(False)
		self.glade_tree.get_widget("button_stop").set_sensitive(True)
		self.glade_tree.get_widget("status_bar").set_status("Profiling started")
		opcontrol_cmd("start")

	def on_stop(self, widget, data=None):
		self.glade_tree.get_widget("button_start").set_sensitive(True)
		self.glade_tree.get_widget("button_stop").set_sensitive(False)
		self.glade_tree.get_widget("status_bar").set_status("Profiling stopped")
		opcontrol_cmd("stop")
		self.refresh_main()

	def on_clear(self, widget, data=None):
		opcontrol_cmd("reset")
		self.refresh_main()

	def on_save(self, widget, data=None):
		print "Save"

	def on_prefs(self, widget, data=None):
		pref_dialog = PrefDialog(parent=self.w_main)
		if pref_dialog.run() == gtk.RESPONSE_OK:
			self.prefs = pref_dialog.prefs
		pref_dialog.destroy()
		self.refresh_main()

	def on_about(self, widget, data=None):
		about_dialog = AboutDialog(parent=self.w_main)

	def on_main_app_delete_event(self, widget, event, data=None):
		# If we return false, we will be sent a destroy signal
		return False

	def on_main_app_destroy(self, widget, data=None):
		gtk.main_quit()

	def main(self):
		# All PyGTK applications must have a gtk.main(). Control ends here
		# and waits for an event to occur (like a key press or mouse event).
		gtk.main()


class AboutDialog:

	d_main = None
	glade_tree = None

	def __init__(self, parent):
		self.glade_tree = gtk.glade.XML("gop.glade", "about_dialog")
		self.d_main = self.glade_tree.get_widget("about_dialog")
		self.d_main.set_transient_for(parent)
		self.glade_tree.signal_autoconnect(self)
		self.d_main.show()


class PrefDialog:

	d_main = None
	glade_tree = None
	prefs = None
	img_liststore = None
	img_treeview = None

	def __init__(self, parent):
		self.prefs = Prefs()
		self.glade_tree = gtk.glade.XML("gop.glade", "pref_dialog")
		self.d_main = self.glade_tree.get_widget("pref_dialog")
		self.d_main.set_transient_for(parent)
		self.glade_tree.signal_autoconnect(self)

		self.img_liststore = gtk.ListStore(str)

		# setup the img treeview
		self.img_treeview = self.glade_tree.get_widget("img_path_tree")
		self.img_treeview.set_model(self.img_liststore)
		column = gtk.TreeViewColumn("Image Path", gtk.CellRendererText(), text=0)
		self.img_treeview.append_column(column)
		if self.prefs.kernel_image_path:
			self.glade_tree.get_widget("kern_file").set_text(self.prefs.kernel_image_path)
		if self.prefs.callgraph_depth:
			self.glade_tree.get_widget("callgraph_depth").set_value(self.prefs.callgraph_depth)
		if self.prefs.image_paths:
			for a in self.prefs.image_paths:
				self.img_liststore.append([a])

	def on_kern_file_button_clicked(self, widget, data=None):
		d = gtk.FileChooserDialog("Kernel Image (vmlinux)", action=gtk.FILE_CHOOSER_ACTION_OPEN,
			parent=self.d_main,
			buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
		if d.run() == gtk.RESPONSE_OK:
			self.glade_tree.get_widget("kern_file").set_text(d.get_filename())
		d.destroy()

	def on_img_path_add_button_clicked(self, widget, data=None):
		d = gtk.FileChooserDialog("Image Path", action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
			parent=self.d_main,
			buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
		if d.run() == gtk.RESPONSE_OK:
			self.img_liststore.append([d.get_filename()])
		d.destroy()

	def on_img_path_remove_button_clicked(self, widget, data=None):
		iter = self.img_treeview.get_selection().get_selected()[1]
		if iter:
			self.img_liststore.remove(iter)

	def run(self):
		retval = self.d_main.run()
		if retval == gtk.RESPONSE_OK:
			self.prefs.kernel_image_path = self.glade_tree.get_widget("kern_file").get_text()
			self.prefs.callgraph_depth = self.glade_tree.get_widget \
				("callgraph_depth").get_value_as_int()
			self.prefs.image_paths = [ r[0] for r in self.img_liststore ]
			self.prefs.save()
		return retval

	def destroy(self):
		return self.d_main.destroy()

class Prefs:

	# gconf stuff
	client = None
	prefs_key = "/apps/gop"
	daemonrc_present = None

	# saved via opcontrol
	kernel_image_path = None
	callgraph_depth = None

	# not in opcontrol
	image_paths = None

	def __init__(self):
		try:
			# TODO fix hardcode
			f = open("/root/.oprofile/daemonrc")
			daemonrc = dict(kv_s.split('=',1) for kv_s in f.read().split())
			if daemonrc["VMLINUX"] and daemonrc["VMLINUX"] != "none":
				self.kernel_image_path = daemonrc["VMLINUX"]
			if daemonrc["CALLGRAPH"].isdigit():
				self.callgraph_depth = int(daemonrc["CALLGRAPH"])
			self.daemonrc_present = True
			f.close()
		except IOError:
			self.daemonrc_present = False
		self.client = gconf.client_get_default()
		self.client.add_dir(self.prefs_key, gconf.CLIENT_PRELOAD_NONE)
		self.image_paths =  self.client.get_list(self.prefs_key+"/image_paths",
												 gconf.VALUE_STRING)

	def save(self):
		if self.kernel_image_path:
			opcontrol_cmd("vmlinux=" + self.kernel_image_path)
		else:
			opcontrol_cmd("no-vmlinux")
		if self.callgraph_depth != None:
			opcontrol_cmd("callgraph=" + str(self.callgraph_depth))
		self.client.set_list(self.prefs_key+"/image_paths", gconf.VALUE_STRING,
							 self.image_paths)

# utility functions

def opcontrol_cmd(command):
	stdout, stderr = subprocess.Popen(["opcontrol", "--" + command],
		stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
	if stderr: print stderr


# If the program is run directly or passed as an argument to the python
# interpreter then create an app instance and show it
if __name__ == "__main__":
	app = MainApp()
	app.main()
